@@ -2,21 +2,21 @@
   mairix - message index builder and finder for maildir folders.
 
  **********************************************************************
- * Copyright (C) Richard P. Curnow  2002,2003,2004,2005
- * 
+ * Copyright (C) Richard P. Curnow  2002,2003,2004,2005,2006
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
- * 
+ *
  **********************************************************************
  */
 
@@ -153,6 +153,12 @@ struct rfc822 {/*{{{*/
 
 typedef char checksum_t[16];
 
+struct message_list {/*{{{*/
+  struct message_list *next;
+  off_t start;
+  size_t len;
+};
+/*}}}*/
 struct mbox {/*{{{*/
   /* If path==NULL, this indicates that the mbox is dead, i.e. no longer
    * exists. */
@@ -171,8 +177,13 @@ struct mbox {/*{{{*/
      new.) */
   int n_old_msgs_valid;
 
+  /* Hold list of new messages and their number.  Number is temporary -
+   * eventually just list walking in case >=2 have to be reattached. */
+  struct message_list *new_msgs;
+  int n_new_msgs;
+
   int n_so_far; /* Used during database load. */
-  
+
   int n_msgs;   /* Number of entries in 'start' and 'len' */
   int max_msgs; /* Allocated size of 'start' and 'len' */
   /* File offset to the start of each message (first line of real header, not to mbox 'From ' line) */
@@ -222,7 +233,8 @@ enum folder_type {/*{{{*/
   FT_MAILDIR,
   FT_MH,
   FT_MBOX,
-  FT_RAW
+  FT_RAW,
+  FT_EXCERPT
 };
 /*}}}*/
 
@@ -291,13 +303,21 @@ void build_message_list(char *folder_base, char *folders, enum folder_type ft,
     struct msgpath_array *msgs, struct globber_array *omit_globs);
 int filter_is_maildir(const char *path, const struct stat *sb);
 int filter_is_mh(const char *path, const struct stat *sb);
-  
+
 /* In rfc822.c */
 struct rfc822 *make_rfc822(char *filename);
 void free_rfc822(struct rfc822 *msg);
-struct rfc822 *data_to_rfc822(struct msg_src *src, char *data, int length);
+enum data_to_rfc822_error {
+  DTR8_OK,
+  DTR8_MISSING_END, /* missing endpoint marker. */
+  DTR8_MULTIPART_SANS_BOUNDARY, /* multipart with no boundary string defined */
+  DTR8_BAD_HEADERS, /* corrupt headers */
+  DTR8_BAD_ATTACHMENT /* corrupt attachment (e.g. no body part) */
+};
+struct rfc822 *data_to_rfc822(struct msg_src *src, char *data, int length, enum data_to_rfc822_error *error);
 void create_ro_mapping(const char *filename, unsigned char **data, int *len);
 void free_ro_mapping(unsigned char *data, int len);
+char *format_msg_src(struct msg_src *src);
 
 /* In tok.c */
 struct toktable *new_toktable(void);
@@ -344,7 +364,7 @@ void write_database(struct database *db, char *filename, int do_integrity_checks
 
 /* In search.c */
 int search_top(int do_threads, int do_augment, char *database_path, char *complete_mfolder, char **argv, enum folder_type ft, int verbose);
-  
+
 /* In stats.c */
 void get_db_stats(struct database *db);
 
@@ -362,4 +382,7 @@ void lock_database(char *path, int forced_unlock);
 void unlock_database(void);
 void unlock_and_exit(int code);
 
+/* In mairix.c */
+void report_error(const char *str, const char *filename);
+
 #endif /* MAIRIX_H */
