@@ -2,21 +2,21 @@
   mairix - message index builder and finder for maildir folders.
 
  **********************************************************************
- * Copyright (C) Richard P. Curnow  2003,2004,2005
- * 
+ * Copyright (C) Richard P. Curnow  2003,2004,2005,2006
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
- * 
+ *
  **********************************************************************
  */
 
@@ -30,6 +30,7 @@
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include "mairix.h"
+#include "from.h"
 #include "fromcheck.h"
 #include "md5.h"
 
@@ -74,7 +75,7 @@ static void append_new_mboxen_to_db(struct database *db, struct extant_mbox *ext
   for (i=N=0; i<n_extant; i++) {
     if (extant_mboxen[i].db_index < 0) N++;
   }
-  
+
   n_reqd = db->n_mboxen + N;
   if (n_reqd > db->max_mboxen) {
     db->max_mboxen = n_reqd;
@@ -167,13 +168,6 @@ static int find_number_intact(struct mbox *mb, char *va, size_t len)/*{{{*/
 }
 /*}}}*/
 
-struct message_list {/*{{{*/
-  struct message_list *next;
-  off_t start;
-  size_t len;
-  checksum_t csum;
-};
-/*}}}*/
 
 static int fromtab_inited = 0;
 static signed char fromtab[256];
@@ -190,8 +184,6 @@ static void init_fromtab(void)/*{{{*/
 }
 /*}}}*/
 
-static signed char fromcheck_table[256];
-
 /* REAL CHECKING : need to see if the line looks like this:
  * From [ <return-path> ] <weekday> <month> <day> <time> [ <timezone> ] <year>
    (from the mutt sources).
@@ -212,7 +204,7 @@ static int looks_like_from_separator(off_t n, char *va, size_t len, int verbose)
     if (verbose) {
       printf("current_state=%d, p=%02x (%1c) ", current_state, (int)(unsigned char)p, ((p>=32)&&(p<=126))?p:'.');
     }
-    current_state = fromcheck_next_state(current_state, (int)fromcheck_table[(int)(unsigned char)p]);
+    current_state = fromcheck_next_state(current_state, (int)fromcheck_char2tok[(int)(unsigned char)p]);
     if (verbose) {
       printf("next_state=%d\n", current_state);
     }
@@ -220,7 +212,7 @@ static int looks_like_from_separator(off_t n, char *va, size_t len, int verbose)
       /* not matched */
       break;
     }
-    if (fromcheck_exitval[current_state] == FROMCHECK_PASS) {
+    if (fromcheck_attr[current_state] == FROMCHECK_PASS) {
       result = 1; /* matched good separator */
       break;
     }
@@ -234,27 +226,6 @@ static int looks_like_from_separator(off_t n, char *va, size_t len, int verbose)
 }
 /*}}}*/
 
-static void init_fromcheck_table()/*{{{*/
-{
-  int i;
-  for (i=0; i<256; i++)   fromcheck_table[i] = -1;
-  for (i='A'; i<='Z'; i++) fromcheck_table[i] = FS_UPPER;
-  for (i='a'; i<='z'; i++) fromcheck_table[i] = FS_LOWER;
-  for (i='0'; i<='9'; i++) fromcheck_table[i] = FS_DIGIT;
-
-  fromcheck_table['+'] = FS_PLUSMINUS;
-  fromcheck_table['-'] = FS_PLUSMINUS;
-  fromcheck_table['@'] = FS_AT;
-  fromcheck_table[':'] = FS_COLON;
-  fromcheck_table['\n'] = FS_LF;
-  fromcheck_table['\r'] = FS_CR;
-  fromcheck_table[' '] = FS_WHITE;
-  fromcheck_table['\t'] = FS_WHITE;
-  fromcheck_table['_'] = FS_OTHEREMAIL;
-  fromcheck_table['.'] = FS_OTHEREMAIL;
-  fromcheck_table['='] = FS_OTHEREMAIL;
-}
-/*}}}*/
 static off_t find_next_from(off_t n, char *va, size_t len)/*{{{*/
 {
   unsigned char c;
@@ -373,7 +344,6 @@ static struct message_list *build_new_message_list(struct mbox *mb, char *va, si
     } else {
       next->len = end_from - start_pos;
     }
-    compute_checksum(va + start_pos, next->len, &next->csum);
     if (!result) {
       result = here = next;
     } else {
@@ -393,30 +363,6 @@ done:
 
 }
 /*}}}*/
-static void append_new_messages(struct mbox *mb, struct message_list *list, int n)/*{{{*/
-{
-  int j;
-  struct message_list *here, *next;
-  int N;
-
-  N = mb->n_old_msgs_valid + n;
-  if (N > mb->max_msgs) {
-    mb->max_msgs = N;
-    mb->start = grow_array(off_t, N, mb->start);
-    mb->len = grow_array(size_t, N, mb->len);
-    mb->check_all = grow_array(checksum_t, N, mb->check_all);
-  }
-  for (j=mb->n_old_msgs_valid, here=list; j<N; j++, here=next) {
-    next = here->next;
-    mb->start[j] = here->start;
-    mb->len[j] = here->len;
-    memcpy(&mb->check_all[j], &here->csum, sizeof(checksum_t));
-
-    free(here);
-  }
-  mb->n_msgs = N;
-}
-/*}}}*/
 static void rescan_mbox(struct mbox *mb, char *va, size_t len)/*{{{*/
 {
   /* We get here if it's determined that
@@ -425,20 +371,16 @@ static void rescan_mbox(struct mbox *mb, char *va, size_t len)/*{{{*/
         since the last mairix run.
   */
 
-  struct message_list *new_messages;
-  int n_new_messages;
-
   /* Find the last message in the box that appears to be intact. */
   mb->n_old_msgs_valid = find_number_intact(mb, va, len);
-  new_messages = build_new_message_list(mb, va, len, &n_new_messages);
-  append_new_messages(mb, new_messages, n_new_messages);
+  mb->new_msgs = build_new_message_list(mb, va, len, &mb->n_new_msgs);
 }
 /*}}}*/
 static void deaden_mbox(struct mbox *mb)/*{{{*/
 {
   mb->n_old_msgs_valid = 0;
   mb->n_msgs = 0;
-  
+
   free(mb->path);
   mb->path = NULL;
 
@@ -555,7 +497,7 @@ static int append_deep(char *path, int base_len, struct stat *sb, struct string_
 
   this_file_matched = append_shallow(path, base_len, sb, list, methods, omit_globs);
   appended_any |= this_file_matched;
-  
+
   if (S_ISDIR(sb->st_mode)) {
     xpath = new_array(char, strlen(path) + 2 + NAME_MAX);
     d = opendir(path);
@@ -664,7 +606,7 @@ static void handle_wild(char *path, int base_len, char *last_comp, struct string
     fprintf(stderr, "WARNING: Folder path %s does not exist\n", temp_path);
   }
 
-  
+
   free(temp_path);
   free(xpath);
   free(gg);
@@ -793,7 +735,7 @@ void glob_and_expand_paths(const char *folder_base,
 
   for (i=0; i<n_in; i++) {
     char *path = paths_in[i];
-    handle_one_path(folder_base, path, &list, methods, omit_globs); 
+    handle_one_path(folder_base, path, &list, methods, omit_globs);
   }
 
   string_list_to_array(&list, n_out, paths_out);
@@ -855,13 +797,12 @@ void build_mbox_lists(struct database *db, const char *folder_base, /*{{{*/
 
   marry_up_mboxen(db, extant_mboxen, n_extant);
 
-  init_fromcheck_table();
-
   /* Now look for new/modified mboxen, find how many of the old messages are
    * still valid and scan the remainder. */
 
   for (i=0; i<db->n_mboxen; i++) {
     struct mbox *mb = &db->mboxen[i];
+    mb->new_msgs = NULL;
     if (mb->path) {
       if ((mb->current_mtime == mb->file_mtime) &&
           (mb->current_size  == mb->file_size)) {
@@ -902,52 +843,121 @@ int add_mbox_messages(struct database *db)/*{{{*/
 {
   int i, j;
   int any_new = 0;
+  int N;
   unsigned char *va;
   int valen;
+  enum data_to_rfc822_error error;
 
   for (i=0; i<db->n_mboxen; i++) {
     struct mbox *mb = &db->mboxen[i];
-    va = NULL; /* lazy mmap */
-    for (j=mb->n_old_msgs_valid; j<mb->n_msgs; j++) {
-      int n;
-      off_t start;
-      size_t len;
-      struct rfc822 *r8;
-      struct msg_src *msg_src;
-      maybe_grow_message_arrays(db);
-      n = db->n_msgs;
-      db->type[n] = MTY_MBOX;
-      db->msgs[n].src.mbox.file_index = i;
-      db->msgs[n].src.mbox.msg_index = j;
+    struct message_list *here, *next;
 
-      if (!va) {
-        create_ro_mapping(mb->path, &va, &valen);
-      }
-      if (!va) {
-        fprintf(stderr, "Couldn't create mapping of file %s\n", mb->path);
-        unlock_and_exit(1);
+    if (mb->new_msgs) {
+      /* Upper bound : we may need to coalesce 2 or more messages if false
+       * matches on From lines have occurred inside MIME encoded body parts. */
+      N = mb->n_old_msgs_valid + mb->n_new_msgs;
+      if (N > mb->max_msgs) {
+        mb->max_msgs = N;
+        mb->start = grow_array(off_t, N, mb->start);
+        mb->len = grow_array(size_t, N, mb->len);
+        mb->check_all = grow_array(checksum_t, N, mb->check_all);
       }
 
-      start = mb->start[j];
-      len   = mb->len[j];
-      msg_src = setup_msg_src(mb->path, start, len);
-      r8 = data_to_rfc822(msg_src, (char *) va + start, len);
-      if (r8) {
-        if (verbose) {
-          printf("Scanning %s[%d] at [%d,%d)\n", mb->path, j, (int)start, (int)(start + len));
+      va = NULL; /* lazy mmap */
+      for (j=mb->n_old_msgs_valid, here=mb->new_msgs; here; j++, here=next) {
+        int n;
+        off_t start;
+        size_t len;
+        struct rfc822 *r8;
+        struct msg_src *msg_src;
+        struct message_list *last, *xx, *xn;
+
+        next = here->next;
+
+        if (!va) {
+          create_ro_mapping(mb->path, &va, &valen);
+        }
+        if (!va) {
+          fprintf(stderr, "Couldn't create mapping of file %s\n", mb->path);
+          unlock_and_exit(1);
         }
-        db->msgs[n].date = r8->hdrs.date;
-        tokenise_message(n, db, r8);
-        free_rfc822(r8);
-      } else {
-        printf("Message in %s at [%d,%d) is misformatted\n", mb->path, (int)start, (int)(start + len));
-      }
 
-      ++db->n_msgs;
-      any_new = 1;
-    }
-    if (va) {
-    	free_ro_mapping(va, valen);
+
+        /* Try to parse the next 'From' -to- 'From' chunk as an rfc822 message.
+         * If we get an unterminated MIME encoding, coalesce the next chunk
+         * onto the current one and try again.  Keep going until it works, or
+         * we run out of chunks.  If we run out, back up to just using the
+         * first chunk and assume it is broken.
+         *
+         * This is to deal with cases such as having a text/plain attachment
+         * that is actually an mbox file in its own right, i.e. will have
+         * embedded '^From ' lines in it.
+         *
+         * 'last' is the last chunk currently in the putative message. */
+        last = here;
+        do {
+          len = last->start + last->len - here->start;
+          msg_src = setup_msg_src(mb->path, here->start, len);
+          r8 = data_to_rfc822(msg_src, (char *) va + here->start, len, &error);
+          if (error == DTR8_MISSING_END) {
+            if (r8) free_rfc822(r8);
+            r8 = NULL;
+            last = last->next; /* Try with another chunk on the end */
+          } else {
+            /* Treat as success */
+            next = last->next;
+            break;
+          }
+        } while (last);
+
+        if (last) {
+          start = mb->start[j] = here->start;
+          mb->len[j] = len;
+          compute_checksum((char *) va + here->start, len, &mb->check_all[j]);
+        } else {
+          /* Faulty message or last message in the file */
+          start = mb->start[j] = here->start;
+          len = mb->len[j] = here->len;
+          compute_checksum((char *) va + here->start, len, &mb->check_all[j]);
+          msg_src = setup_msg_src(mb->path, start, len);
+          r8 = data_to_rfc822(msg_src, (char *) va + start, len, &error);
+          if (error == DTR8_MISSING_END) {
+            fprintf(stderr, "Can't find end boundary in multipart message %s\n",
+                format_msg_src(msg_src));
+          }
+        }
+
+        /* Release all the list entries in the range [here,next) (inclusive) */
+        for (xx=here; xx!=next; xx=xn) {
+          xn = xx->next;
+          free(xx);
+        }
+
+        /* Only do this once a valid rfc822 structure has been obtained. */
+        maybe_grow_message_arrays(db);
+        n = db->n_msgs;
+        db->type[n] = MTY_MBOX;
+        db->msgs[n].src.mbox.file_index = i;
+        db->msgs[n].src.mbox.msg_index = j;
+
+        if (r8) {
+          if (verbose) {
+            printf("Scanning %s[%d] at [%d,%d)\n", mb->path, j, (int)start, (int)(start + len));
+          }
+          db->msgs[n].date = r8->hdrs.date;
+          tokenise_message(n, db, r8);
+          free_rfc822(r8);
+        } else {
+          printf("Message in %s at [%d,%d) is misformatted\n", mb->path, (int)start, (int)(start + len));
+        }
+
+        ++db->n_msgs;
+        any_new = 1;
+      }
+      mb->n_msgs = j;
+      if (va) {
+        free_ro_mapping(va, valen);
+      }
     }
   }
   return any_new;
@@ -1015,8 +1025,7 @@ void decode_mbox_indices(unsigned int index, unsigned int *mb, unsigned int *msg
   *msg = (index & 0xffff);
 }
 /*}}}*/
-
-int verify_mbox_size_constraints(struct database *db)
+int verify_mbox_size_constraints(struct database *db)/*{{{*/
 {
   int i;
   int fail;
@@ -1035,5 +1044,5 @@ int verify_mbox_size_constraints(struct database *db)
   if (fail) return 0;
   else      return 1;
 }
-
+/*}}}*/
 
