@@ -6,20 +6,21 @@
  * rfc2047 decode Copyright (C) Mikael Ylikoski 2002
  * gzip mbox support Copyright (C) Ico Doornekamp 2005
  * gzip mbox support Copyright (C) Felipe Gustavo de Almeida 2005
- * 
+ * bzip2 mbox support Copyright (C) Paramjit Oberoi 2005
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
- * 
+ *
  **********************************************************************
  */
 
@@ -35,6 +36,9 @@
 #ifdef USE_GZIP_MBOX
 #  include <zlib.h>
 #endif
+#ifdef USE_BZIP_MBOX
+#  include <bzlib.h>
+#endif
 
 struct DLL {/*{{{*/
   struct DLL *next;
@@ -56,7 +60,7 @@ static void enqueue(void *head, void *x)/*{{{*/
 /*}}}*/
 
 enum encoding_type {/*{{{*/
-  ENC_UNKNOWN, 
+  ENC_UNKNOWN,
   ENC_NONE,
   ENC_BINARY,
   ENC_7BIT,
@@ -130,7 +134,7 @@ static void splice_header_lines(struct line *header)/*{{{*/
 static int audit_header(struct line *header)/*{{{*/
 {
   /* Check for obvious broken-ness
-   * 1st line has no leading spaces, single word then colon 
+   * 1st line has no leading spaces, single word then colon
    * following lines have leading spaces or single word followed by colon
    * */
   struct line *x;
@@ -387,11 +391,21 @@ static enum encoding_type decode_encoding_type(char *e)/*{{{*/
 static char *copy_string_start_end_unquote(char *start, char *end)/*{{{*/
 {
   char *result, *p, *q;
-  result = new_array(char, 1 + (end - start));
-  for (p=result, q=start; q < end; q++) {
-    if (*q != '"') *p++ = *q;
+  char *squote;
+  squote = (char *) memchr(start, '"', end - start);
+  if (squote) {
+    /* Quoted string; only do the portion between the quotes. */
+    result = new_array(char, 1 + (end - squote));
+    for (p=result, q=squote+1; q < end; q++) {
+      if (*q == '"') break;
+      *p++ = *q;
+    }
+    *p = 0;
+  } else {
+    result = new_array(char, 1 + (end - start));
+    memcpy(result, start, end - start);
+    result[end - start] = 0;
   }
-  *p = 0;
   return result;
 }
 /*}}}*/
@@ -403,7 +417,7 @@ static void parse_content_type(char *hdrline, struct content_type_header *result
   result->major = NULL;
   result->minor = NULL;
   result->boundary = NULL;
-    
+
   p = hdrline;
   while (*p && isspace(*(unsigned char *)p)) p++;
   for (q=p+1; *q && (*q != '/'); q++) ;
@@ -413,7 +427,7 @@ static void parse_content_type(char *hdrline, struct content_type_header *result
     result->major = new_array(char, 1 + (q - p));
     for (s=result->major; p<q;) *s++ = *p++;
     *s = 0;
-   
+
     p = q + 1;
     for (q=p+1; *q && !isspace(*(unsigned char *) q) && (*q != ';'); q++) ;
     result->minor = new_array(char, 1 + (q - p));
@@ -437,14 +451,14 @@ static void parse_content_type(char *hdrline, struct content_type_header *result
       if (!*value) break;
 
       /* Find next semicolon, or end of line, or whitespace (if not a quoted RHS) */
-      for (semi = value+1; 
+      for (semi = value+1;
            *semi && ((*value == '"') || (!isspace(*(unsigned char *)semi))) && (*semi != ';');
            semi++) {}
 
       if (!strncasecmp(name, "boundary", 8)) {
         result->boundary = copy_string_start_end_unquote(value, semi);
       }
-      
+
       semi = strchr(semi, ';'); /* in case value string was ended by whitespace */
 
     }
@@ -454,7 +468,7 @@ static void parse_content_type(char *hdrline, struct content_type_header *result
     result->major = new_array(char, 1 + (q - p));
     for (s=result->major; p<q;) *s++ = *p++;
     *s = 0;
-   
+
     /* Assume text will be plain */
     if (match_string(result->major, "text")) {
       result->minor = new_string("plain");
@@ -485,7 +499,7 @@ static char *unencode_data(char *input, int input_len, char *enc, int *output_le
   enum encoding_type encoding;
   char *result, *end_result;
   char *end_input;
- 
+
   encoding = decode_encoding_type(enc);
   end_input = input + input_len;
 
@@ -580,7 +594,7 @@ static char *unencode_data(char *input, int input_len, char *enc, int *output_le
   return result;
 }
 /*}}}*/
-static char *format_msg_src(struct msg_src *src)/*{{{*/
+char *format_msg_src(struct msg_src *src)/*{{{*/
 {
   static char *buffer = NULL;
   static int buffer_len = 0;
@@ -598,7 +612,7 @@ static char *format_msg_src(struct msg_src *src)/*{{{*/
         buffer = new_array(char, len);
         buffer_len = len;
       }
-      sprintf(buffer, "%s[%d,%d)", src->filename, 
+      sprintf(buffer, "%s[%d,%d)", src->filename,
           (int) src->start, (int) (src->start + src->len));
       result = buffer;
       break;
@@ -628,7 +642,7 @@ static int split_and_splice_header(struct msg_src *src, char *data, struct line
         int line_length = eol - sol;
         char *line_text = new_array(char, 1 + line_length);
         struct line *new_header;
-        
+
         strncpy(line_text, sol, line_length);
         line_text[line_length] = '\0';
         new_header = new(struct line);
@@ -659,9 +673,16 @@ static int split_and_splice_header(struct msg_src *src, char *data, struct line
 /*}}}*/
 
 /* Forward prototypes */
-static void do_multipart(struct msg_src *src, char *input, int input_len, char *boundary, struct attachment *atts);
-  
-static void do_body(struct msg_src *src, char *body_start, int body_len, char *content_type, char *content_transfer_encoding, struct attachment *atts)/*{{{*/
+static void do_multipart(struct msg_src *src, char *input, int input_len,
+    char *boundary, struct attachment *atts,
+    enum data_to_rfc822_error *error);
+
+/*{{{ do_body() */
+static void do_body(struct msg_src *src,
+    char *body_start, int body_len,
+    char *content_type, char *content_transfer_encoding,
+    struct attachment *atts,
+    enum data_to_rfc822_error *error)
 {
   char *decoded_body;
   int decoded_body_len;
@@ -672,12 +693,11 @@ static void do_body(struct msg_src *src, char *body_start, int body_len, char *c
     struct content_type_header ct;
     parse_content_type(content_type, &ct);
     if (!strcasecmp(ct.major, "multipart")) {
-      do_multipart(src, decoded_body, decoded_body_len, ct.boundary, atts);
-
+      do_multipart(src, decoded_body, decoded_body_len, ct.boundary, atts, error);
       /* Don't need decoded body any longer - copies have been taken if
        * required when handling multipart attachments. */
       free(decoded_body);
-
+      if (error && (*error == DTR8_MISSING_END)) return;
     } else {
       /* unipart */
       struct attachment *new_att;
@@ -697,7 +717,7 @@ static void do_body(struct msg_src *src, char *body_start, int body_len, char *c
       }
 
       if (new_att->ct == CT_MESSAGE_RFC822) {
-        new_att->data.rfc822 = data_to_rfc822(src, decoded_body, decoded_body_len);
+        new_att->data.rfc822 = data_to_rfc822(src, decoded_body, decoded_body_len, error);
         free(decoded_body); /* data no longer needed */
       } else {
         new_att->data.normal.len = decoded_body_len;
@@ -722,7 +742,10 @@ static void do_body(struct msg_src *src, char *body_start, int body_len, char *c
   }
 }
 /*}}}*/
-static void do_attachment(struct msg_src *src, char *start, char *after_end, struct attachment *atts)/*{{{*/
+/*{{{ do_attachment() */
+static void do_attachment(struct msg_src *src,
+    char *start, char *after_end,
+    struct attachment *atts)
 {
   /* decode attachment and add to attachment list */
   struct line header, *x, *nx;
@@ -730,7 +753,7 @@ static void do_attachment(struct msg_src *src, char *start, char *after_end, str
   int body_len;
   char *content_type, *content_transfer_encoding;
   if (split_and_splice_header(src, start, &header, &body_start) < 0) {
-    fprintf(stderr, "Giving up on attachment with bad header in %s\n", 
+    fprintf(stderr, "Giving up on attachment with bad header in %s\n",
         format_msg_src(src));
     return;
   }
@@ -751,7 +774,8 @@ static void do_attachment(struct msg_src *src, char *start, char *after_end, str
     }
   } else {
     body_len = after_end - body_start;
-    do_body(src, body_start, body_len, content_type, content_transfer_encoding, atts);
+    /* Ignore errors in nested body parts. */
+    do_body(src, body_start, body_len, content_type, content_transfer_encoding, atts, NULL);
   }
 
   /* Free header memory */
@@ -765,7 +789,12 @@ static void do_attachment(struct msg_src *src, char *start, char *after_end, str
   if (content_transfer_encoding) free(content_transfer_encoding);
 }
 /*}}}*/
-static void do_multipart(struct msg_src *src, char *input, int input_len, char *boundary, struct attachment *atts)/*{{{*/
+/*{{{ do_multipart() */
+static void do_multipart(struct msg_src *src,
+    char *input, int input_len,
+    char *boundary,
+    struct attachment *atts,
+    enum data_to_rfc822_error *error)
 {
   char *normal_boundary, *end_boundary;
   char *b0, *b1, *be;
@@ -776,6 +805,7 @@ static void do_multipart(struct msg_src *src, char *input, int input_len, char *
   if (!boundary) {
     fprintf(stderr, "Can't process multipart message %s with no boundary string\n",
         format_msg_src(src));
+    if (error) *error = DTR8_MULTIPART_SANS_BOUNDARY;
     return;
   }
 
@@ -785,7 +815,7 @@ static void do_multipart(struct msg_src *src, char *input, int input_len, char *
 
   strcpy(normal_boundary, "--");
   strcat(normal_boundary, boundary);
-  
+
   strcpy(end_boundary, "--");
   strcat(end_boundary, boundary);
   strcat(end_boundary, "--");
@@ -794,13 +824,17 @@ static void do_multipart(struct msg_src *src, char *input, int input_len, char *
   /* Scan input to look for boundary markers */
   be = strstr(input, end_boundary);
   if (!be) {
-    fprintf(stderr, "Can't find end boundary in multipart message %s\n",
-        format_msg_src(src));
-    be = strchr(input, 0); /* tolerate missing end boundary */
+    if (error) {
+      *error = DTR8_MISSING_END;
+      return;
+    } else {
+      /* soldier on as best we can */
+      be = strchr(input, 0);
+    }
   }
 
   line_after_b0 = input;
-  
+
   do {
     int boundary_ok;
     start_b1_search_from = line_after_b0;
@@ -818,7 +852,7 @@ static void do_multipart(struct msg_src *src, char *input, int input_len, char *
           break;
         }
       }
-      
+
       looking_at_end_boundary = (b1 == be);
       boundary_ok = 1;
       if ((b1 > input) && (*(b1-1) != '\n')) boundary_ok = 0;
@@ -887,7 +921,7 @@ static time_t parse_rfc822_date(char *date_string)/*{{{*/
   } else if (tm.tm_year >= 1900) {
     tm.tm_year -= 1900;
   }
-  
+
   while (isdigit(*s)) s++;
   while (*s && isspace(*s)) s++;
   if (!*s) goto tough_cheese;
@@ -905,7 +939,10 @@ tough_cheese:
   return (time_t) -1; /* default value */
 }
 /*}}}*/
-struct rfc822 *data_to_rfc822(struct msg_src *src, char *data, int length)/*{{{*/
+/*{{{ data_to_rfc822() */
+struct rfc822 *data_to_rfc822(struct msg_src *src,
+    char *data, int length,
+    enum data_to_rfc822_error *error)
 {
   struct rfc822 *result;
   char *body_start;
@@ -913,7 +950,8 @@ struct rfc822 *data_to_rfc822(struct msg_src *src, char *data, int length)/*{{{*
   struct line *x, *nx;
   char *content_type, *content_transfer_encoding;
   int body_len;
-  
+
+  if (error) *error = DTR8_OK; /* default */
   result = new(struct rfc822);
   init_headers(&result->hdrs);
   result->atts.next = result->atts.prev = &result->atts;
@@ -923,6 +961,7 @@ struct rfc822 *data_to_rfc822(struct msg_src *src, char *data, int length)/*{{{*
       fprintf(stderr, "Giving up on message %s with bad header\n",
           format_msg_src(src));
     }
+    if (error) *error = DTR8_BAD_HEADERS;
     return NULL;
   }
 
@@ -948,7 +987,7 @@ struct rfc822 *data_to_rfc822(struct msg_src *src, char *data, int length)/*{{{*
 
   /* Process body */
   body_len = length - (body_start - data);
-  do_body(src, body_start, body_len, content_type, content_transfer_encoding, &result->atts);
+  do_body(src, body_start, body_len, content_type, content_transfer_encoding, &result->atts, error);
 
   /* Free header memory */
   for (x=header.next; x!=&header; x=nx) {
@@ -961,7 +1000,7 @@ struct rfc822 *data_to_rfc822(struct msg_src *src, char *data, int length)/*{{{*
   if (content_transfer_encoding) free(content_transfer_encoding);
 
   return result;
-  
+
 }
 /*}}}*/
 
@@ -971,60 +1010,166 @@ struct rfc822 *data_to_rfc822(struct msg_src *src, char *data, int length)/*{{{*
 
 int data_alloc_type;
 
-#ifdef USE_GZIP_MBOX
+#if USE_GZIP_MBOX || USE_BZIP_MBOX
+
 #define SIZE_STEP (8 * 1024 * 1024)
-static int is_gzip(const char *filename) {/*{{{*/
+
+#define COMPRESSION_NONE 0
+#define COMPRESSION_GZIP 1
+#define COMPRESSION_BZIP 2
+
+static int get_compression_type(const char *filename) {/*{{{*/
   size_t len = strlen(filename);
-  int ptr = len - 3;
+  int ptr;
+
+#ifdef USE_GZIP_MBOX
+  ptr = len - 3;
   if (len > 3 && strncasecmp(filename + ptr, ".gz", 3) == 0) {
-    return 1;
-  } else {
+    return COMPRESSION_GZIP;
+  }
+#endif
+
+#ifdef USE_BZIP_MBOX
+  ptr = len - 4;
+  if (len > 3 && strncasecmp(filename + ptr, ".bz2", 4) == 0) {
+    return COMPRESSION_BZIP;
+  }
+#endif
+
+  return COMPRESSION_NONE;
+}
+/*}}}*/
+
+static int is_compressed(const char *filename) {/*{{{*/
+  return (get_compression_type(filename) != COMPRESSION_NONE);
+}
+/*}}}*/
+
+struct zFile {/*{{{*/
+  union {
+    /* Both gzFile and BZFILE* are defined as void pointers
+     * in their respective header files.
+     */
+#ifdef USE_GZIP_MBOX
+    gzFile gzf;
+#endif
+#ifdef USE_BZIP_MBOX
+    BZFILE *bzf;
+#endif
+    void *zptr;
+  };
+  int type;
+};
+/*}}}*/
+
+static struct zFile * xx_zopen(const char *filename, const char *mode) {/*{{{*/
+  struct zFile *zf = new(struct zFile);
+
+  zf->type = get_compression_type(filename);
+  switch (zf->type) {
+#ifdef USE_GZIP_MBOX
+    case COMPRESSION_GZIP:
+      zf->gzf = gzopen(filename, "rb");
+      break;
+#endif
+#ifdef USE_BZIP_MBOX
+    case COMPRESSION_BZIP:
+      zf->bzf = BZ2_bzopen(filename, "rb");
+      break;
+#endif
+    default:
+      zf->zptr = NULL;
+      break;
+  }
+
+  if (!zf->zptr) {
+    free(zf);
     return 0;
   }
+
+  return zf;
+}
+/*}}}*/
+static void xx_zclose(struct zFile *zf) {/*{{{*/
+  switch (zf->type) {
+#ifdef USE_GZIP_MBOX
+    case COMPRESSION_GZIP:
+      gzclose(zf->gzf);
+      break;
+#endif
+#ifdef USE_BZIP_MBOX
+    case COMPRESSION_BZIP:
+      BZ2_bzclose(zf->bzf);
+      break;
+#endif
+    default:
+      zf->zptr = NULL;
+      break;
+  }
+  free(zf);
+}
+/*}}}*/
+static int xx_zread(struct zFile *zf, void *buf, int len) {/*{{{*/
+  switch (zf->type) {
+#ifdef USE_GZIP_MBOX
+    case COMPRESSION_GZIP:
+      return gzread(zf->gzf, buf, len);
+      break;
+#endif
+#ifdef USE_BZIP_MBOX
+    case COMPRESSION_BZIP:
+      return BZ2_bzread(zf->bzf, buf, len);
+      break;
+#endif
+    default:
+      return 0;
+      break;
+  }
 }
 /*}}}*/
 #endif
+
 void create_ro_mapping(const char *filename, unsigned char **data, int *len)/*{{{*/
 {
   struct stat sb;
   int fd;
 
-#ifdef USE_GZIP_MBOX
-  gzFile gzf;
+#if USE_GZIP_MBOX || USE_BZIP_MBOX
+  struct zFile *zf;
 #endif
 
-  if (stat(filename, &sb) < 0) 
+  if (stat(filename, &sb) < 0)
   {
-    perror("Could not stat");
+    report_error("stat", filename);
     *data = NULL;
     return;
   }
-  
-#ifdef USE_GZIP_MBOX
-  if(is_gzip(filename)) {
+
+#if USE_GZIP_MBOX || USE_BZIP_MBOX
+  if(is_compressed(filename)) {
     if(verbose) {
     	fprintf(stderr, "Decompressing %s...\n", filename);
     }
-  
-    gzf = gzopen(filename, "rb");
-    if (!gzf) {
+
+    zf = xx_zopen(filename, "rb");
+    if (!zf) {
       fprintf(stderr, "Could not open %s\n", filename);
       *data = NULL;
       *len = 0;
       return;
     }
     *data = new_array(unsigned char, SIZE_STEP);
-    *len = gzread(gzf, *data, SIZE_STEP);
+    *len = xx_zread(zf, *data, SIZE_STEP);
     if (*len >= SIZE_STEP) {
       int extra_bytes_read;
       do {
         *data = grow_array(unsigned char, *len + SIZE_STEP, *data);
-        extra_bytes_read = gzread(gzf, *data + *len, SIZE_STEP);
+        extra_bytes_read = xx_zread(zf, *data + *len, SIZE_STEP);
         *len += extra_bytes_read;
       } while (extra_bytes_read > 0);
     }
-    gzclose(gzf);
-    
+    xx_zclose(zf);
+
     if(*len > 0) {
       *data = grow_array(unsigned char, *len, *data);
     	data_alloc_type = ALLOC_MALLOC;
@@ -1032,17 +1177,17 @@ void create_ro_mapping(const char *filename, unsigned char **data, int *len)/*{{
       free(*data);
       data_alloc_type = ALLOC_NONE;
     }
-        
+
     return;
   }
-#endif /* USE_GZIP_MBOX */
-  
+#endif /* USE_GZIP_MBOX || USE_BZIP_MBOX */
+
   *len = sb.st_size;
   if (*len == 0) {
     *data = NULL;
     return;
   }
-  
+
   if (!S_ISREG(sb.st_mode)) {
     *data = NULL;
     return;
@@ -1051,16 +1196,16 @@ void create_ro_mapping(const char *filename, unsigned char **data, int *len)/*{{
   fd = open(filename, O_RDONLY);
   if (fd < 0)
   {
-    perror("Could not open");
+    report_error("open", filename);
     *data = NULL;
     return;
   }
 
   *data = (unsigned char *) mmap(0, *len, PROT_READ, MAP_SHARED, fd, 0);
   if (close(fd) < 0)
-    perror("close");
+    report_error("close", filename);
   if (*data == MAP_FAILED) {
-    perror("rfc822:mmap");
+    report_error("rfc822:mmap", filename);
     *data = NULL;
     return;
   }
@@ -1109,11 +1254,12 @@ struct rfc822 *make_rfc822(char *filename)/*{{{*/
     struct msg_src *src;
     /* Now process the data */
     src = setup_msg_src(filename);
-    result = data_to_rfc822(src, (char *) data, len);
+    /* For one message per file, ignore missing end boundary condition. */
+    result = data_to_rfc822(src, (char *) data, len, NULL);
 
     free_ro_mapping(data, len);
   }
-  
+
   return result;
 }
 /*}}}*/
@@ -1122,7 +1268,7 @@ void free_rfc822(struct rfc822 *msg)/*{{{*/
   struct attachment *a, *na;
 
   if (!msg) return;
-  
+
   if (msg->hdrs.to) free(msg->hdrs.to);
   if (msg->hdrs.cc) free(msg->hdrs.cc);
   if (msg->hdrs.from) free(msg->hdrs.from);
@@ -1130,7 +1276,7 @@ void free_rfc822(struct rfc822 *msg)/*{{{*/
   if (msg->hdrs.message_id) free(msg->hdrs.message_id);
   if (msg->hdrs.in_reply_to) free(msg->hdrs.in_reply_to);
   if (msg->hdrs.references) free(msg->hdrs.references);
-  
+
   for (a = msg->atts.next; a != &msg->atts; a = na) {
     na = a->next;
     if (a->ct == CT_MESSAGE_RFC822) {
@@ -1197,7 +1343,7 @@ static void show_rfc822(struct rfc822 *msg, int indent)/*{{{*/
 int main (int argc, char **argv)/*{{{*/
 {
   struct rfc822 *msg;
-  
+
   if (argc < 2) {
     fprintf(stderr, "Need a path\n");
     unlock_and_exit(2);
@@ -1206,7 +1352,7 @@ int main (int argc, char **argv)/*{{{*/
   msg = make_rfc822(argv[1]);
   show_rfc822(msg, 0);
   free_rfc822(msg);
-  
+
   /* Print out some stuff */
 
   return 0;
