@@ -2,21 +2,21 @@
   mairix - message index builder and finder for maildir folders.
 
  **********************************************************************
- * Copyright (C) Richard P. Curnow  2002,2003,2004,2005
- * 
+ * Copyright (C) Richard P. Curnow  2002,2003,2004,2005,2006
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
  * published by the Free Software Foundation.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
- * 
+ *
  **********************************************************************
  */
 
@@ -105,7 +105,7 @@ static int substring_match_0(unsigned long *a, unsigned long hit, int left_ancho
   char *p;
   unsigned long r0;
   unsigned long anchor, anchor1;
-  
+
   r0 = ~0;
   got_hit = 0;
   anchor = 0;
@@ -453,7 +453,7 @@ static void parse_size_range(char *size_expr, int *has_start, int *start, int *h
   char *x = size_expr;
   char *dash;
   int len;
-  
+
   if (*x == ':') x++;
   len = strlen(x);
   dash = strchr(x, '-');
@@ -491,7 +491,7 @@ static void find_size_matches_in_table(struct read_db *db, char *size_expr, char
   int start, end;
   int has_start, has_end, start_cond, end_cond;
   int i;
-  
+
   start = end = -1; /* avoid compiler warning about uninitialised variables. */
   parse_size_range(size_expr, &has_start, &start, &has_end, &end);
   if (has_start && has_end) {
@@ -518,7 +518,7 @@ static void find_date_matches_in_table(struct read_db *db, char *date_expr, char
   int has_start, has_end, start_cond, end_cond;
   int i;
   int status;
-  
+
   status = scan_date_string(date_expr, &start, &has_start, &end, &has_end);
   if (status) {
     unlock_and_exit (2);
@@ -543,27 +543,29 @@ static void find_date_matches_in_table(struct read_db *db, char *date_expr, char
 }
 /*}}}*/
 
-static char *mk_maildir_path(int token, char *output_dir, int is_in_new)/*{{{*/
+static char *mk_maildir_path(int token, char *output_dir, int is_in_new, const char *flags)/*{{{*/
 {
-  char *result; 
+  char *result;
   char uniq_buf[48];
   int len;
+  int flag_len;
 
   len = strlen(output_dir) + 64; /* oversize */
-  result = new_array(char, len);
+  flag_len = flags ? strlen(flags) : 0;
+  result = new_array(char, len + flag_len);
   strcpy(result, output_dir);
   strcat(result, is_in_new ? "/new/" : "/cur/");
   sprintf(uniq_buf, "123456789.%d.mairix", token);
   strcat(result, uniq_buf);
-  if (!is_in_new) {
-    strcat(uniq_buf, ":2,S");
+  if (flags) {
+    strcat(result, flags);
   }
   return result;
 }
 /*}}}*/
 static char *mk_mh_path(int token, char *output_dir)/*{{{*/
 {
-  char *result; 
+  char *result;
   char uniq_buf[8];
   int len;
 
@@ -593,6 +595,23 @@ static int looks_like_maildir_new_p(const char *p)/*{{{*/
   }
 }
 /*}}}*/
+static char *get_maildir_flags(char *p)/*{{{*/
+{
+  char *x;
+  x = p + strlen(p);
+  for (x--; x>p; x--) {
+    switch (*x) {
+      case ':':
+        return x;
+      case '/':
+        return NULL;
+      default:
+        break;
+    }
+  }
+  return NULL;
+}
+/*}}}*/
 static void create_symlink(char *link_target, char *new_link)/*{{{*/
 {
   if (symlink(link_target, new_link) < 0) {
@@ -697,6 +716,16 @@ static void try_copy_to_path(struct read_db *db, int msg_index, char *target_pat
   return;
 }
 /*}}}*/
+static struct msg_src *setup_mbox_msg_src(char *filename, off_t start, size_t len)/*{{{*/
+{
+  static struct msg_src result;
+  result.type = MS_MBOX;
+  result.filename = filename;
+  result.start = start;
+  result.len = len;
+  return &result;
+}
+/*}}}*/
 static int do_search(struct read_db *db, char **args, char *output_path, int show_threads, enum folder_type ft, int verbose)/*{{{*/
 {
   char *colon, *start_words;
@@ -726,15 +755,15 @@ static int do_search(struct read_db *db, char **args, char *output_path, int sho
    * hit2 gathers the tokens  or'ed with ,
    * hit3 gathers the separate args and'ed with <gap>
    * */
-  
-  
+
+
   /* Everything matches until proven otherwise */
   memset(hit3, 1, db->n_msgs);
-  
+
   while (*args) {
     /* key is a single argument, separate args are and-ed together */
     key = *args++;
-    
+
     memset(hit2, 0, db->n_msgs);
     memset(hit1, 1, db->n_msgs);
 
@@ -747,7 +776,7 @@ static int do_search(struct read_db *db, char **args, char *output_path, int sho
     do_size = 0;
     do_path = 0;
     do_msgid = 0;
-    
+
     colon = strchr(key, ':');
 
     if (colon) {
@@ -799,7 +828,7 @@ static int do_search(struct read_db *db, char **args, char *output_path, int sho
       int negate;
       int had_orsep;
       int max_errors;
-      
+
       orsep = strchr(start_words, '/');
       andsep  = strchr(start_words, ',');
       had_orsep = 0;
@@ -846,7 +875,7 @@ static int do_search(struct read_db *db, char **args, char *output_path, int sho
       } else {
         left_anchor = 0;
       }
-      
+
       equal = strchr(word, '=');
       if (equal) {
         *equal = 0;
@@ -901,7 +930,7 @@ static int do_search(struct read_db *db, char **args, char *output_path, int sho
         }
         memset(hit1, 1, db->n_msgs);
       }
-      
+
       free(orig_word);
 
     } while (*start_words);
@@ -947,10 +976,12 @@ static int do_search(struct read_db *db, char **args, char *output_path, int sho
               {
                 char *target_path;
                 char *message_path;
+                char *start_flags;
                 int is_in_new;
                 message_path = db->data + db->path_offsets[i];
                 is_in_new = looks_like_maildir_new_p(message_path);
-                target_path = mk_maildir_path(i, output_path, is_in_new);
+                start_flags = get_maildir_flags(message_path);
+                target_path = mk_maildir_path(i, output_path, is_in_new, start_flags);
                 create_symlink(message_path, target_path);
                 free(target_path);
                 ++n_hits;
@@ -958,7 +989,14 @@ static int do_search(struct read_db *db, char **args, char *output_path, int sho
               break;
             case DB_MSG_MBOX:
               {
-                char *target_path = mk_maildir_path(i, output_path, 0);
+                /* For messages stored in a MBOX, we have no idea what the
+                 * flags are, since we don't parse those headers.  However, if
+                 * we're creating a maildir mfolder, it's most likely that the
+                 * mboxen are being used for archiving old stuff, so we guess
+                 * :2,S as the required suffix.   In any case, messages in
+                 * /cur/ are required to have some kind of flags suffix, so we
+                 * have to put something there.  */
+                char *target_path = mk_maildir_path(i, output_path, 0, ":2,S");
                 try_copy_to_path(db, i, target_path);
                 free(target_path);
                 ++n_hits;
@@ -1060,6 +1098,69 @@ static int do_search(struct read_db *db, char **args, char *output_path, int sho
         }
       }
       break;
+/*}}}*/
+    case FT_EXCERPT:/*{{{*/
+      for (i=0; i<db->n_msgs; i++) {
+        if (hit3[i]) {
+          struct rfc822 *parsed = NULL;
+          switch (db->msg_type[i]) {
+            case DB_MSG_FILE:
+              {
+                char *filename;
+                ++n_hits;
+                printf("---------------------------------\n");
+                filename = db->data + db->path_offsets[i];
+                printf("%s\n", filename);
+                parsed = make_rfc822(filename);
+              }
+              break;
+            case DB_MSG_MBOX:
+              {
+                unsigned int mbix, msgix;
+                int start, len, after_end;
+                unsigned char *mbox_start, *msg_start;
+                int mbox_len, msg_len;
+                int mbox_index;
+
+                start = db->mtime_table[i];
+                len   = db->size_table[i];
+                after_end = start + len;
+                ++n_hits;
+                printf("---------------------------------\n");
+                decode_mbox_indices(db->path_offsets[i], &mbix, &msgix);
+                printf("mbox:%s [%d,%d)\n", db->data + db->mbox_paths_table[mbix], start, after_end);
+
+                get_validated_mbox_msg(db, i, &mbox_index, &mbox_start, &mbox_len, &msg_start, &msg_len);
+                if (msg_start) {
+                  enum data_to_rfc822_error error;
+                  struct msg_src *msg_src;
+                  msg_src = setup_mbox_msg_src(db->data + db->mbox_paths_table[mbix], start, msg_len);
+                  parsed = data_to_rfc822(msg_src, (char *) msg_start, msg_len, &error);
+                }
+                if (mbox_start) {
+                  free_ro_mapping(mbox_start, mbox_len);
+                }
+              }
+              break;
+            case DB_MSG_DEAD:
+              break;
+          }
+
+          if (parsed) {
+            char datebuf[64];
+            struct tm *thetm;
+            if (parsed->hdrs.to)      printf("  To:      %s\n", parsed->hdrs.to);
+            if (parsed->hdrs.cc)      printf("  Cc:      %s\n", parsed->hdrs.cc);
+            if (parsed->hdrs.from)    printf("  From:    %s\n", parsed->hdrs.from);
+            if (parsed->hdrs.subject) printf("  Subject: %s\n", parsed->hdrs.subject);
+            thetm = gmtime(&parsed->hdrs.date);
+            strftime(datebuf, sizeof(datebuf), "%a, %d %b %Y", thetm);
+            printf("  Date:     %s\n", datebuf);
+            free_rfc822(parsed);
+          }
+        }
+      }
+      break;
 /*}}}*/
     default:
       assert(0);
@@ -1070,7 +1171,7 @@ static int do_search(struct read_db *db, char **args, char *output_path, int sho
   free(hit1);
   free(hit2);
   free(hit3);
-  if (ft != FT_RAW) {
+  if ((ft != FT_RAW) && (ft != FT_EXCERPT)) {
     printf("Matched %d messages\n", n_hits);
   }
   fflush(stdout);
@@ -1081,7 +1182,7 @@ static int do_search(struct read_db *db, char **args, char *output_path, int sho
             "Matches were found in mbox folders but the message checksums failed.\n"
             "You may need to run mairix in indexing mode then repeat your search.\n");
   }
-  
+
   /* Return error code 1 to the shell if no messages were matched. */
   return (n_hits == 0) ? 1 : 0;
 }
@@ -1117,7 +1218,7 @@ static void maybe_create_maildir(char *path)/*{{{*/
 {
   char *subdir, *tailpos;
   int len;
-  
+
   if (!directory_exists_remove_other(path)) {
     create_dir(path);
   }
@@ -1247,6 +1348,7 @@ int search_top(int do_threads, int do_augment, char *database_path, char *comple
       /* Nothing to do */
       break;
     case FT_RAW:
+    case FT_EXCERPT:
       break;
     default:
       assert(0);
@@ -1265,6 +1367,7 @@ int search_top(int do_threads, int do_augment, char *database_path, char *comple
         clear_mbox_folder(complete_mfolder);
         break;
       case FT_RAW:
+      case FT_EXCERPT:
         break;
       default:
         assert(0);
